#!/usr/bin/perl -w

package RPM::Source::Convert::Generic2ALT;

use strict;
use warnings;
use Carp;

sub alt_generic_fix {
    my $spec=shift;
    my @sections=$spec->get_sections();
    my $cleanroot_pattern='^\s*(rm|%{__rm}) -rf (\$RPM_BUILD_ROOT|\%\{?rpm_build_root\}?|\%\{?buildroot\}?)\s*$';
    &alt_generic_check_ahead($spec,$sections[0]);
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    $sec->exclude('^(Packager|Vendor|Distribution|Build[Rr]oot):');
	    &alt_generic_cleanup_headers($sec);
	} elsif ($t eq 'description') {
	    &alt_generic_fix_desc($sec);
	} elsif ($t eq 'files') {
	    &alt_generic_cleanup_files($sec);
	} elsif ($t eq 'prep' || $t eq 'build' || $t eq 'install') {
	    $sec->exclude($cleanroot_pattern);
	} elsif ($t eq 'clean') {
	    $sec->exclude($cleanroot_pattern);
	    if ($sec->is_empty()) {
		# ?
	    }
	    # hack! due to unsafe iteration!
	    #$sec->empty();$sec->behead();
	    $spec->del_section('clean');
	} elsif ($t eq 'check') {
	    # not supported in alt rpm :(
	    # TODO: move to the end of %build
	    warn 'TODO: fix %check section properly!';
	    #$sec->empty();$sec->behead();
	    $spec->del_section('check');
	} elsif ($t eq 'changelog') {
	    $sec->empty();
	} elsif ($t eq 'post') {
	    &alt_generic_convert_alternatives($spec,$sec);
	} elsif (($t eq 'postun') or ($t eq 'preun')) {
	    &alt_generic_convert_alternatives($spec,$sec);
	} elsif (($t eq 'triggerin') or ($t eq 'triggerun') or ($t eq 'triggerpostin') or ($t eq 'triggerposun')) {
	    # not good, no state :(
	    #&alt_generic_convert_alternatives($spec,$sec);
	}
    }
}

our @ALT_ALLOWED_GROUPS=(
'Accessibility',
'Archiving/Backup',
'Archiving/Cd burning',
'Archiving/Compression',
'Archiving/Other',
'Books/Computer books',
'Books/Faqs',
'Books/Howtos',
'Books/Literature',
'Books/Other',
'Communications',
'Databases',
'Development/C',
'Development/C++',
'Development/Databases',
'Development/Debuggers',
'Development/Documentation',
'Development/Functional',
'Development/GNOME and GTK+',
'Development/Haskell',
'Development/Java',
'Development/KDE and QT',
'Development/Kernel',
'Development/Lisp',
'Development/ML',
'Development/Objective-C',
'Development/Other',
'Development/Perl',
'Development/Python',
'Development/Ruby',
'Development/Scheme',
'Development/Tcl',
'Documentation',
'Editors',
'Education',
'Emulators',
'File tools',
'Games/Adventure',
'Games/Arcade',
'Games/Boards',
'Games/Cards',
'Games/Educational',
'Games/Other',
'Games/Puzzles',
'Games/Sports',
'Games/Strategy',
'Graphical desktop/Enlightenment',
'Graphical desktop/FVWM based',
'Graphical desktop/GNOME',
'Graphical desktop/GNUstep',
'Graphical desktop/Icewm',
'Graphical desktop/KDE',
'Graphical desktop/Motif',
'Graphical desktop/Other',
'Graphical desktop/Rox',
'Graphical desktop/Sawfish',
'Graphical desktop/Window Maker',
'Graphical desktop/XFce',
'Graphics',
'Monitoring',
'Networking/Chat',
'Networking/DNS',
'Networking/File transfer',
'Networking/FTN',
'Networking/IRC',
'Networking/Instant messaging',
'Networking/Mail',
'Networking/News',
'Networking/Other',
'Networking/Remote access',
'Networking/WWW',
'Office',
'Publishing',
'Sciences/Astronomy',
'Sciences/Biology',
'Sciences/Chemistry',
'Sciences/Computer science',
'Sciences/Geosciences',
'Sciences/Mathematics',
'Sciences/Medicine',
'Sciences/Other',
'Sciences/Physics',
'Security/Antivirus',
'Security/Networking',
'Shells',
'Sound',
'System/Base',
'System/Configuration/Boot and Init',
'System/Configuration/Hardware',
'System/Configuration/Networking',
'System/Configuration/Other',
'System/Configuration/Packaging',
'System/Configuration/Printing',
'System/Fonts/Console',
'System/Fonts/True type',
'System/Fonts/Type1',
'System/Fonts/X11 bitmap',
'System/Internationalization',
'System/Kernel and hardware',
'System/Libraries',
'System/Legacy libraries',
'System/Servers',
'System/Servers/ZProducts',
'System/X11',
'System/XFree86',
'Terminals',
'Text tools',
'Toys',
'Video',
);

our %IS_ALLOWED_GROUP = map {$_=>1} @ALT_ALLOWED_GROUPS;

our %ALT_GROUP_MAP = (
'Text Editors', 'Editors',
'Development/Tools', 'Development/Other',
'System Environment/Base', 'System/Base',
# in aspell
#'Applications/Text', 'System/Internationalization',
' ', ' ',
    ); 

sub alt_generic_cleanup_headers {
    my $section=shift;
    my @out;
    foreach (@{$section->get_body()}) {
	if (/^Group:(\s*)(.+\S)\s*$/) {
	    my $gspace=$1;
	    my $group=$2;
	    if ($ALT_GROUP_MAP{$group}) {
		push @out, "Group:$gspace$ALT_GROUP_MAP{$group}\n";
	    } else {
		push @out, $_;
	    }
	} elsif (/^\s*((?:BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts)\s*(?:\(\w+\))?)\s*:\s*(.*\S)\s*$/) {
	    my $Tag=$1;
	    my $filtered = &alt_filter_pkg_list($2);
	    push @out, "$Tag: $filtered\n" if ($filtered);
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

our %ALTDEPSKIP = map {$_ => 1} qw!
%{__make}
/bin/rm /bin/ln
!;

our %ALTDEPSUBST = (
'%{_sbindir}/update-alternatives' => 'alternatives >= 0:0.4',
'/usr/sbin/update-alternatives' => 'alternatives >= 0:0.4',
'%{_sbindir}/alternatives' => 'alternatives >= 0:0.4',
'/usr/sbin/alternatives' => 'alternatives >= 0:0.4',
'alsa-lib' => 'libalsa',
'alsa-lib-devel' => 'libalsa-devel',
'apr-devel' => 'libapr1-devel',
'at-spi' => 'libat-spi',
'at-spi-devel' => 'libat-spi-devel',
'automake14' => 'automake_1.4',
'automake16' => 'automake_1.6',
'automake17' => 'automake_1.7',
'automake18' => 'automake_1.8',
'automake19' => 'automake_1.9',
'cairo-devel' => 'libcairo-devel',
'cairo' => 'libcairo',
'commons-digester' => 'jakarta-commons-digester',
'freetype-devel' => 'libfreetype-devel',
'giflib' => 'libungif',
'giflib-devel' => 'libungif-devel',
'gnome-vfs2-devel' => 'gnome-vfs-devel',
'gnome-vfs2' => 'gnome-vfs',
'mono-core' => 'mono',
'mesa-libGL-devel' => 'libmesa-devel',
'mesa-libGLU-devel' => 'libmesa-devel',
'mozilla-nspr-devel' => 'libnspr-devel',
'nspr-devel' => 'libnspr-devel',
'openssl-devel' => 'libssl-devel',
'pulseaudio-libs-devel' => 'libpulseaudio-devel',
'xorg-x11-fonts-misc' => 'fonts-bitmap-misc',
'xorg-x11-fonts-Type1' => 'fonts-type1-xorg',
'xorg-x11-server-Xvfb' => 'xorg-x11-xvfb',
' ' => ' ',
);

sub alt_filter_pkg_list {
    $_=$_[0];
    # hack around add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    s/\>\s+=/>=/g;
    s/\<\s+=/<=/g;
    my @in = split /[\s,]+/, $_;
    my @out;
    while (@in) {
	$_ = shift @in;
	if ($ALTDEPSKIP{$_}) {
	    &read_tag_version(\@in);
	    next;
	} elsif ($ALTDEPSUBST{$_}) {
	    push @out, $ALTDEPSUBST{$_};
	} else {
	    push @out, $_;
	}
    }
    return join(" ",@out);
};


# is macro %name used in spec before Name:,Version: tags
# it is not allowed in alt
sub alt_generic_check_ahead {
    my ($spec,$section)=@_;
    my @out;
    my $is_name_passed=0;
    my $is_name_ahead=0;
    foreach (@{$section->get_body()}) {
	m/^\s*Name:/ and $is_name_passed=1; #and print STDERR "passed $_\n";
	$is_name_ahead=1 if m/\%\{name\}/ and $is_name_passed==0;
#print STDERR "AHEAD $_\n";
	$is_name_ahead=1 if m/\%name(?:[^\w\d_]|$)/ and $is_name_passed==0;
#print STDERR "ahead $_\n";
    }
    if ($is_name_ahead) {
	$section->unshift_body('%define name '.$spec->get_section('package','')->get_tag('Name')."\n");
	$section->unshift_body('%define version '.$spec->get_section('package','')->get_tag('Version')."\n");
	print STDERR "fixed ahead!!!\n";
    }
}

sub alt_generic_fix_desc {
    my $section=shift;
    my @out;
    foreach (@{$section->get_body()}) {
# some distro use iso in description. alt use ascii only.
	    #tr//e/;
	    tr//...eooeaiaoucououaaidieaayvnieouabEOOEAIAOUCOUOUAAIDIEAAYVNIEOUAB/;
	    # % in description
	    s,%(\s|$),%% ,g;
	    push @out, $_;
    }
    $section->set_body(\@out);
}

sub alt_generic_cleanup_files {
    my $section=shift;
    my @out;
    my $need_docdir=0;
    foreach (@{$section->get_body()}) {
	s/^\s*\%attr\(-,root,root\)\s*//;
	#if (/^\s*\%defattr\(0644,root,root,0755\)\s*$/
	if (/^\s*\%defattr\((?:0644|-),root,root,(?:0755|-)\)\s*$/
	    or /^\s*\%defattr\((?:0664|-),root,root,(?:0755|-)\)\s*$/
	    or /^\s*\%defattr\((?:0644|-),root,root\)\s*$/
	    ) {
	    # throw away
	} elsif (/^\s*\%defattr\(0755,root,root,0755\)\s*$/) {
	    # dangerous: as in antlr may cause lost 755 rights in /usr/bin
	    push @out, "#".$_;
	} elsif (/^\s*\%doc\s*\%\{_docdir\}\s*$/) {
	    push @out, '%doc %{_docdir}/*'."\n";
	} elsif (m!^\s*\%doc\s*\%\{?_docdir\}?/\%\{?name\}?-\%\{?version\}?/\S+!) {
	    push @out, $_;
	    $need_docdir=1;
	} else {
	    push @out, $_;
	}
    }
    if ($need_docdir) {
	push @out, '# hack; explicitly added docdir if not owned'."\n";
	push @out, '%doc %dir %{_docdir}/%{name}-%{version}'."\n";
    }
    $section->set_body(\@out);
}

sub alt_generic_convert_alternatives {
    my $spec=shift;
    my $section=shift;
    my @out;
    my $ptr = $section->get_body();
    while (@$ptr>0) {
	my $line = shift @$ptr;
	if ($line=~/\s*#/) {
	    push @out, $line;
	} elsif ($line=~s,^(\s*)(?:(?:\%{_sbindir}|/usr/sbin)/)?(?:update-)?alternatives --(\w+),,) {
	    my $preline=$1;
	    my $updcmd=$2;
	    push @out, &_convert_single_alternative($spec,$section,$ptr,$preline,$updcmd,$line);
	} elsif ($line=~s,^(\s*)(?:(?:\%{_sbindir}|/usr/sbin)/)?(?:update-)?alternatives\s+\\\n$,,) {
	    my $preline=$1;
	    $line = shift @$ptr;
	    if ($line=~s/^\s*--(\w+)//) {
		my $updcmd=$1;
		push @out, &_convert_single_alternative($spec,$section,$ptr,$preline,$updcmd,$line);
	    } else {
		unshift @$ptr, $line;
		push @out, $preline."# unknown update-alternatives\n";
	    }
	} else {
	    push @out, $line;
	}
    }
    $section->set_body(\@out);
}

sub _convert_single_alternative {
    my ($spec,$section,$ptr,$preline,$updcmd,$line)=@_;
    $line=&read_multiline($line, $ptr);
    # the end is abandoned :(
    if ($updcmd eq "remove") {
	my $name=&read_shell_token(\$line);
	# pre 0.4 alt alternatives
	#return $preline.'%unregister_alternatives '.&get_alternatives_file_name($spec,$section,$name)."\n";
	return $preline.":\n";
    } elsif ($updcmd eq "install") {
	my $link=&read_shell_token(\$line);
	my $name=&read_shell_token(\$line);
	my $path=&read_shell_token(\$line);
	my $priority=&read_shell_token(\$line);
	my @alternatives=($link,$path,$priority);
	&read_space(\$line);
	while ($line=~/--slave/) {
	    &read_shell_token(\$line);
	    my $slink=&read_shell_token(\$line);
	    my $sname=&read_shell_token(\$line);
	    my $spath=&read_shell_token(\$line);
	    push @alternatives, ($slink,$spath,$path);
	}
	&create_alternatives($spec,$section, &get_alternatives_file_name($spec,$section,$name),\@alternatives);
	return $preline.":\n";
    } else {
	warn "unknown update-alternatives --$updcmd";
	return $preline."# unknown update-alternatives --$updcmd\n";
    }
}

sub create_alternatives {
    my ($spec, $section, $alternatives_file_name,$alternativesptr) = @_;
    my @text=('install -d $RPM_BUILD_ROOT/%_altdir; cat >$RPM_BUILD_ROOT/%_altdir/'.$alternatives_file_name.'<<EOF
');
    while (@$alternativesptr>0) {
	my $provided=shift(@$alternativesptr);
	my $alternative=shift(@$alternativesptr);
	my $weight=shift(@$alternativesptr);
	push @text, $provided."\t".$alternative."\t".$weight."\n";
    }
    push @text, "EOF\n";
    $spec->get_section('install')->push_body(@text);
    my $filesec=$spec->get_section('files',$section->get_package());
    if ($filesec) {
	$filesec->unshift_body('%_altdir/'.$alternatives_file_name."\n");
    } else {
	warn "WARNING: section %files for [".$section->get_package()."] not found: can't write alternatives found in ".$section->get_type()."!";    }
}

sub get_alternatives_file_name {
    my ($spec,$section,$name)=@_;
    return $name.'_'.$section->get_package_expanded();
}

sub read_tag_version {
    my $ptr=shift;
    return if ($#{$ptr}<1);
    if ($ptr->[0] =~/^(?:<|>|=|!=|>=|<=)$/) {
	return (shift @$ptr, shift @$ptr);
    }
    return;
}

sub read_multiline {
    my ($line, $ptr) =@_;
    while (@$ptr>0 && $line=~s/\\[\r\n]+?$//) {
	chomp $line;
#	print "MULTILINE:$line\n";
	$line.=shift(@$ptr);
    }
    return $line;
}

sub read_shell_token {
    my ($lineptr) =@_;
    if ($$lineptr=~s/^\s*(\S+)//) {
	return $1;
    }
    return "";
}

sub read_space {
    my ($lineptr) =@_;
    $$lineptr=~s/^\s*//;
}

1;
